package fr.diwaly.minicraft;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.Random;
import java.util.Scanner;

import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.block.BlockFace;
import org.bukkit.block.Chest;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;

public class GenTreasurePiece extends AbstractGen{

	private MiniVolcano volcano;
	
	private int nbLoot;
	private int[] items;
	private int[] itemsStack;
	//private int[] itemsRate;
	private Block chestB;
	private ItemStack[] chestI = new ItemStack[27];
	
	private GenEnterTunnel genEnterTunnel;
	
	public GenTreasurePiece(MiniVolcano volcano, Block ref,  BlockFace faceX, BlockFace faceY,  BlockFace faceZ) {
		super(ref, faceX, faceY, faceZ);		
		this.volcano = volcano;			
		chestB = getChest();
		createLoot(getFile());
		loadLoot(getFile());
	}
	
	public void init(){				
		// CHEST
		if(chestB.getType() == Material.CHEST)
			getInventory().clear();
		// contour
		Build.RECTANGLE(ref, faceX, faceY, faceZ, 12, 8, 9, Material.BEDROCK); 
		// pice
		Block r = ref.getRelative(faceX).getRelative(faceY).getRelative(faceZ);
		Build.RECTANGLE(r, faceX, faceY, faceZ, 10, 6, 7, Material.AIR, Material.MOSSY_COBBLESTONE); 
		// porte
		r = r.getRelative(faceX.getOppositeFace(),3).getRelative(faceZ,2);
		Build.RECTANGLE(r, faceX, faceY, faceZ, 5, 4, 3, Material.AIR, Material.MOSSY_COBBLESTONE); 
		// Tunnel d'entr
		genEnterTunnel = new GenEnterTunnel(r, faceX, faceY, faceZ);
		genEnterTunnel.init();
		// ouvrir la porte
		r = r.getRelative(faceZ).getRelative(faceY);
		Build.RECTANGLE(r, faceX, faceY, faceZ, 5, 2, 1, Material.AIR);
		// CHEST
		initChest();
	}
	
	public void clean(){
		// CHEST
		getInventory().clear();
		// Remplac contour
		Build.RECTANGLE(ref, faceX, faceY, faceZ, 12, 8, 9, Material.MOSSY_COBBLESTONE); 
		// Efface l'interieur
		Block r = ref.getRelative(faceX).getRelative(faceY).getRelative(faceZ);
		Build.RECTANGLE(r, faceX, faceY, faceZ, 10, 6, 7, Material.AIR, Material.MOSSY_COBBLESTONE); 
		// Ajout lave en haut
		r = r.getRelative(faceX).getRelative(faceZ);
		Block haut = r.getRelative(faceY,5);
		Build.RECTANGLE(haut, faceX, faceY, faceZ, 8, 1, 5, Material.LAVA, Material.LAVA);
		// Efface le sol
		r = r.getRelative(faceY);
		Build.RECTANGLE(r, faceX, faceY.getOppositeFace(), faceZ, 8, 3, 5, Material.AIR, Material.AIR);
		// Efface la porte
		r = ref.getRelative(faceX).getRelative(faceY).getRelative(faceZ);
		r = r.getRelative(faceX.getOppositeFace(),3).getRelative(faceZ,2);
		Build.RECTANGLE(r, faceX, faceY, faceZ, 5, 4, 3, Material.LAVA); 
		// Tunnel d'entr
		if(genEnterTunnel != null)
			genEnterTunnel.clean();
	}
	
	public void close(){
		Block close = ref.getRelative(faceY).getRelative(faceZ,3);
		Build.RECTANGLE(close, faceX, faceY, faceZ, 1, 4, 3, Material.BEDROCK, Material.BEDROCK);
		close = close.getRelative(faceX).getRelative(faceY).getRelative(faceZ);
		Build.RECTANGLE(close, faceX, faceY, faceZ, 1, 2, 1, Material.BEDROCK);
	}
	
	public Block getChest(){
		return ref.getRelative(faceX,8).getRelative(faceY,2).getRelative(faceZ,4);
		
	}
	public boolean isTheChest(Block block){
		if(block.getType() == Material.CHEST)
			return block == chestB;
		return false;
	}
	
	private void initChest(){
		int r;
		int nbItems = 5;
		Random rand = new Random();		
		
		for(int i = 0 ; i < 27 ; i++){
			if(i < nbItems){
				r = rand.nextInt(nbLoot);
				chestI[i] = new ItemStack(items[r],itemsStack[r]);
			}else{
				chestI[i] = new ItemStack(0);
			}			
		}
		chestB.setType(Material.CHEST);
		getInventory().setContents(chestI);
	}
	
	public Inventory getInventory(){
		return ((Chest)chestB.getState()).getInventory();
	}
	
	private void createLoot(File file){
		if(!file.exists()){
			try{
				BufferedWriter writer = new BufferedWriter(new FileWriter(file));
				writer.append("nbLoot: 13");
				writer.newLine();
				writer.append("1 "+Material.CHAINMAIL_HELMET);
				writer.newLine();
				writer.append("1 "+Material.CHAINMAIL_CHESTPLATE);
				writer.newLine();
				writer.append("1 "+Material.CHAINMAIL_LEGGINGS);
				writer.newLine();
				writer.append("1 "+Material.CHAINMAIL_BOOTS);
				writer.newLine();
				writer.append("20 "+Material.OBSIDIAN);
				writer.newLine();
				writer.append("5 "+Material.SLIME_BALL);
				writer.newLine();				
				writer.append("32 "+Material.COOKIE);
				writer.newLine();
				writer.append("5 "+Material.GOLDEN_APPLE);
				writer.newLine();
				writer.append("10 "+Material.MUSHROOM_SOUP);
				writer.newLine();
				writer.append("3 "+Material.CAKE);
				writer.newLine();
				writer.append("10 "+Material.APPLE);
				writer.newLine();				
				writer.append("64 "+Material.ARROW);
				writer.newLine();
				writer.append("1 "+Material.SADDLE);
				writer.close();
			} catch (Exception e) {
				Plugin.log("Error create loot: "+e.getMessage());
			}
		}
	}
	
	private void loadLoot(File file){
		try {
			if(file.exists()){
				// Lire le fichier des properties
				Scanner scan = new Scanner(file);
				 scan.next();
				nbLoot = scan.nextInt();
				items = new int[nbLoot];
				itemsStack = new int[nbLoot];
				int i = 0;
				while(scan.hasNext()){
					itemsStack[i] = scan.nextInt();
					String mat = scan.next();
					items[i] = Material.getMaterial(mat).getId();    
					i++;
				}
				scan.close();
			}
		} catch (Exception e) {
			Plugin.log("Error load loot: "+e.getMessage());			
		}		
	}
	
	public File getFile(){
		return getFile(volcano.name);
	}
	static final public File getFile(String name){
		File dir = new File(Plugin.dataFolder,"minitrap");
		dir.mkdir();
		File file = new File(dir, name);
		return file;
	}
}
