package fr.diwaly.minicraft;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Scanner;

import org.bukkit.ChatColor;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Player;
import org.bukkit.event.entity.EntityDeathEvent;
import org.bukkit.event.player.PlayerRespawnEvent;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.PlayerInventory;

public class MiniPlayer {

	static private HashMap<String, MiniPlayer> players = new HashMap<String, MiniPlayer>();
	
	public Player player;
	public String name;
	public SpongeRob rob;
	public MiniBand band;
	static public ArrayList<MiniPlayer> listKarma;
	public boolean karma ;
	private ItemStack[] inventory;
	private ItemStack[] armorContents;
	
	public MiniPlayer(Player p) {
		name = p.getName();
		player = p;
		players.put(name, this);
		rob  = new SpongeRob(this);
		band = new MiniBand(this);
		listKarma = new ArrayList<MiniPlayer>();
		karma = false;
		load();
	}

	public boolean onKarmaDeath(){
		MiniVolcano volcano = Plugin.giveWorldVolcano(player.getWorld());
		if(volcano != null){
			Location loc = player.getLocation();
			if(volcano.trap.genKarma.inZoneKarmaChunk(loc.getBlockX(), loc.getBlockZ())){
				listKarma.add(this);
				karma = true;
				Plugin.server.broadcastMessage(ChatColor.DARK_PURPLE+player.getDisplayName()+" has a good karma.");
			}
		}
		if(karma){
			inventory = player.getInventory().getContents();
			armorContents = player.getInventory().getArmorContents();
			player.sendMessage(ChatColor.LIGHT_PURPLE+"You were reincarnated.");
			return true;
		}
		return false;
	}
	public void onKarmaRespawn(){
		if(inventory != null && karma){
			PlayerInventory inv = player.getInventory();
			inv.setContents(inventory);
			inv.setArmorContents(armorContents);
		}
	}
	public boolean goodKarma(){
		return karma;
	}
	
	public void sendMessage(String msg){
		player.sendMessage(msg);
	}
	
	static public MiniPlayer get(String name){
		return players.get(name);
	}
	
	static public void onEntityDeath(EntityDeathEvent event) {
		Entity entity = event.getEntity();
		if(entity instanceof Player){
			MiniPlayer player = MiniPlayer.get(((Player) entity).getName());
			if(player != null && player.onKarmaDeath())
				event.getDrops().clear();	
		}
	}
	
	static public void onPlayerRespawn(PlayerRespawnEvent event) {
		MiniPlayer player = get(event.getPlayer().getName());
		if(player != null)
			player.onKarmaRespawn();
	}
	
	static public void onPlayerLogin(Player player) {
		MiniVolcano v = Plugin.giveWorldVolcano(player.getWorld());
		if(v!=null){
			if(players.containsKey(player.getName())){
				MiniPlayer miniPlayer = MiniPlayer.get(player.getName());
				if(miniPlayer != null)
					miniPlayer.player = player;
			}else {
				new MiniPlayer(player);
			}
			
		}
	}
	
	public void save(){		
		try {
			BufferedWriter writer = new BufferedWriter(new FileWriter(getFile()));
			writer.append("chef: "+band.chef);
			writer.newLine();
			writer.append("world: "+rob.world.getName());
			writer.newLine();
			writer.append(String.valueOf(rob.X));
			writer.newLine();
			writer.append(String.valueOf(rob.Y));
			writer.newLine();
			writer.append(String.valueOf(rob.Z));
			writer.close();
		} catch (IOException e) {
			Plugin.log("Error Save MiniPlayer: "+e.getMessage());
		}
	}
	public void load(){
		File file = getFile();
		if(file.exists()){
			try {
				Scanner scan = new Scanner(file);
				scan.next();
				band.chef = scan.next();
				scan.next();
				String world = scan.next();
				int x = scan.nextInt();
				int y = scan.nextInt();
				int z = scan.nextInt();
				scan.close();
				rob.set(Plugin.server.getWorld(world).getBlockAt(x, y, z));
				Plugin.log("Load player "+name);
			} catch (FileNotFoundException e) {
				Plugin.log("Error load MiniPlayer: "+e.getMessage());
			}
		}
	}
	private File getFile(){
		File dir = new File(Plugin.dataFolder,"players");
		dir.mkdir();
		File file = new File(dir,name);
		return file;
	}
	
	static public void checkPlayers(World w){
		for (Player player : w.getPlayers()) {
			if(MiniPlayer.get(player.getName()) == null)
				new MiniPlayer(player);
		}
	}
	
}
